Package: weatherStats
Title: Airport Weather Station Statistics
Version: 0.1
Authors@R: person("Peter","Hoff",email="peter.hoff@duke.edu",role=c("aut","cre"))
Description: Download daily weather data recorded at airport weather stations using the National Centers for Environmental Information (NCEI) API <https://www.ncei.noaa.gov/support/access-search-service-api-user-documentation>. 
Depends: R (>= 3.5.0)
License: GPL (>= 3)
RoxygenNote: 7.3.3
LazyData: true
NeedsCompilation: no
Packaged: 2026-04-23 12:50:54 UTC; pdh10
Author: Peter Hoff [aut, cre]
Maintainer: Peter Hoff <peter.hoff@duke.edu>
Repository: CRAN
Date/Publication: 2026-04-28 18:10:09 UTC
Built: R 4.5.2; ; 2026-05-01 17:20:59 UTC; unix

# Generated by roxygen2: do not edit by hand

export(c2f)
export(f2c)
export(isMonth)
export(tpdata)
export(tpplot)
export(weatherStats)
importFrom(grDevices,rgb)
importFrom(graphics,abline)
importFrom(graphics,axis)
importFrom(graphics,par)
importFrom(graphics,points)
importFrom(graphics,segments)
importFrom(graphics,text)
importFrom(utils,read.csv)

#' @title C to F temperature conversion   
#' 
#' @description Celsius to Fahrenheit temperature coversion  
#' 
#' @param x temperature in Celsius 
#'
#' @return temperature in Fahrenheit
#'
#' @author Peter Hoff  
#'
#' @export
c2f<-function(x){ 32 + 1.8 * x }  

#' @title F to C temperature conversion   
#' 
#' @description Fahrenheit to Celsius temperature coversion  
#' 
#' @param y temperature in Fahrenheit
#' 
#' @return temperature in Celsius 
#'
#' @author Peter Hoff   
#' 
#' @export
f2c <- function(y) { (y - 32) / 1.8 } 

#' @title Weather Station Statistics
#' 
#' @description Download weather data from the National Centers for Environmental Information (NCEI) using their API. 
#' 
#' @param FAAID Federal Aviation Administration (FAA) station ID 
#' @param year year for which data are to be downloaded
#' @param startDate start date in YY-MM-DD format
#' @param endDate end date in YY-MM-DD format 
#' @param trimNA remove days with no data 
#' @param maxTries maximum number of query attempts to the NCEI server 
#' @param tpause time in seconds to pause between API requests 
#' @param verbose a logical value indicating whether to report download progress
#' 
#' @return a data frame with daily weather data 
#'
#' @author Peter Hoff
#' 
#' @examples   
#' \donttest{
#' RDU<-weatherStats("RDU",startDate="2024-01-01",endDate="2025-12-31") 
#' SEA<-weatherStats("SEA",year=2025)   
#' }
#'  
#' dim(RDU) 
#' dim(SEA) 
#' colnames(SEA)
#'  
#' @export
weatherStats<-function(FAAID,
  year=NULL,startDate=NULL,endDate=NULL,trimNA=TRUE,maxTries=5,tpause=3,verbose=TRUE){ 

  ## dates 
  if(is.null(year) & is.null(startDate)){ year<-format(Sys.time(),"%Y") }
  if(is.null(startDate)){ startDate=paste0(year,"-01-01") } 
  if(is.null(endDate)){ endDate=paste0(substr(startDate,1,4),"-12-31") }

  ## convert FAA ID to station ID
  stationID<-STATIONS$GHCND[which(STATIONS$FAA==FAAID)]

  ## construct API query  
  query<-paste0("https://www.ncei.noaa.gov/access/services/data/v1?dataset=daily-summaries",
                "&stations=",stationID, 
                "&startDate=",startDate,
                "&endDate=",endDate,
                "&includeStationLocation=true&units=metric")  

  ## read data 
  numTries<-1 
  dat<-try(suppressWarnings(read.csv(query)),silent=TRUE)
  while(attr(dat,"class")=="try-error" & numTries<maxTries){  
    if(verbose){ cat("Query attempt",numTries,"failed\n") }
    Sys.sleep(tpause) 
    dat<-try(suppressWarnings(read.csv(query)),silent=TRUE)
    numTries<-numTries+1 
  } 

  if(attr(dat,"class")=="try-error"){ dat<-NULL } 

  if(!is.null(dat)){  
    ## change station id to FAA id
    dat$STATION<-FAAID  

    ## remove dates with no data
    if(trimNA){ dat<-dat[,apply(!is.na(dat),2,any )] } 
  }

  dat
}

#' @title Temperature and precipitation data 
#' 
#' @description Extract and combine temperature and precipitation data  
#' 
#' @param \ldots either a list or sequence of data matrices obtained from `weatherStats`
#'
#' @return a data frame 
#'
#' @author Peter Hoff
#'
#' @examples 
#' W<-tpdata(RDU,SEA) 
#'  
#' colnames(W) 
#' dim(W) 
#' 
#' tpplot(W) 
#'
#' @export
tpdata <- function(...)
{
  WDAT<-list(...)  

  TP<-NULL 
  for (k in 1:length(WDAT)) {  

    W<-WDAT[[k]]      

    ## min, avg and max temperature  
    if ( !any(colnames(W)=="TAVG") ) { W$TAVG<-NA }     
    temp<-W[, c("TMIN","TAVG","TMAX")]  
    if ( any(is.na(temp[, 2])) ) { 
      i <- which(is.na(temp[, 2]))
      temp[i, 2] <- .5 * (temp[i, 1] + temp[i, 3]) 
    }

    ## precipitation
    prcp<-W[, c("PRCP")]  

    ## non-missing data frame
    tpdata<-data.frame(STATION = W$STATION, DATE = W$DATE,
                       TMIN = temp$TMIN, TAVG = temp$TAVG, TMAX = temp$TMAX,
                       PRCP = prcp)  
    i <- apply(tpdata, 1, function (x) { !any(is.na(x)) } ) 
    tpdata <- tpdata[i, ] 
  
    TP <- rbind(TP ,tpdata)
  } 

  TP <- TP[order(TP$DATE), ] 
  TP <- TP[order(TP$STATION), ] 

TP
}

#' @title Month-specific date extraction 
#' 
#' @description Extract indices of a particular month from ISO 8601 format
#' 
#' @param dates a vector of character dates in IS( 8601 format
#' @param month an integer from 1 to 12
#' 
#' @return a vector of matching indices
#'
#' @author Peter Hoff   
#' 
#' @export
isMonth<-function(dates,month){  
  as.numeric(substring(dates,6,7)) == month
}

#' RDU weather dataset 
#'
#' Two years of weather data at RDU. 
#'
#' @format A data frame with dates along the rows, variables along the columns.  
#' 
#' @source Derived from original data at https://example.org
"RDU" 

#' SEA weather dataset 
#'
#' One year of weather data at SEA. 
#'
#' @format A data frame with dates along the rows, variables along the columns.  
#' 
#' @source Downloaded via the NCEI API <https://www.ncei.noaa.gov/support/access-search-service-api-user-documentation>. 
"SEA"

#' Airport Weather Station Statistics
#' 
#' Download daily weather data recorded at airport weather stations using 
#' the National Centers for Environmental Information (NCEI) API 
#' <https://www.ncei.noaa.gov/support/access-search-service-api-user-documentation>. 
#' 
#' 
#' @name weatherStats-package 
#' 
#' @author Peter Hoff <peter.hoff@@duke.edu> 
#' 
#' @keywords internal
#' 
#' @examples 
#' \donttest{
#' RDU<-weatherStats("RDU",startDate="2024-01-01",endDate="2025-12-31") 
#' SEA<-weatherStats("SEA",year=2025)   
#' }
#' W<-tpdata(RDU,SEA) 
#'  
#' colnames(W) 
#' dim(W) 
#' tpplot(W) 
#'
#' @importFrom grDevices rgb
#' @importFrom graphics abline axis par points segments text 
#' @importFrom utils read.csv 
"_PACKAGE" 

#' @title Temperature and precipitation plot 
#' 
#' @description Plot temperature and precipitation data  
#' 
#' @param TPDATA A data frame with temperature and precipitation data 
#' @param units "F" or "C" 
#' @param rc make multiple plots along columns ("col") or rows ("row") 
#' 
#' @return Invisibly returns \code{NULL}.
#' 
#' @author Peter Hoff
#' 
#' @examples 
#' W<-tpdata(RDU,SEA) 
#'  
#' colnames(W) 
#' dim(W) 
#' 
#' tpplot(W) 
#' 
#' @export
tpplot<-function(TPDATA,units="F",rc="col"){ 

  ## rain colors
  blue<-pmin(TPDATA$PRCP/20,1) 
  rcols<-rgb(1-blue,1-blue,1,alpha=1) 

  ## temp colors 
  tgr<-(TPDATA$TAVG+12)/(38+12) 
  tgr<-pmax(0,pmin(1,tgr))  
  tcols<-rgb(tgr,0,1-tgr,alpha=1) 

  ## stations
  stations<-unique(TPDATA$STATION) 

  ## convert to F  
  ylab<-expression(paste("temperature(",degree*C,")"))
  if(units=="F"){ 
    TPDATA[,3:5]<- 32 + TPDATA[,3:5] * 1.8 
    ylab<-expression(paste("temperature(",degree*F,")"))
  } 
  ylab<-c(ylab,rep("",length(stations)-1)) 
  yaxt<-c("s",rep("n",length(stations)-1)) 

  ## save old value for `par` and revert to it on exit
  oldpar <- par(no.readonly = TRUE) 
  on.exit(par(oldpar)) 

  ## plot data for each station 
  if(rc=="row") { par(mfrow=c(1,length(stations)),mar=c(3,3,1.25,1),mgp=c(1.75,.75,0),
                  oma=c(0,0,0,0)) } 

  if(rc=="col"){  par(mfrow=c(length(stations),1),mar=c(3,3,1.25,1),mgp=c(1.75,.75,0),
                  oma=c(0,0,0,0)) } 

  for(i in 1:length(stations)){  
    istat<-which(TPDATA$STATION==stations[i])
    TPS<-TPDATA[istat,]

    plot(c(-3,nrow(TPS)+3),range(TPDATA[,c("TMIN","TMAX")] ),type="n",
    xlab=TPS$STATION[1],xaxt="n",ylab=ylab[i],yaxt=yaxt[i],xaxs="i") 

    ## boundaries 
    month<-as.numeric(substr(TPS$DATE,6,7) )    
    year<-as.numeric(substr(TPS$DATE,1,4) )   
    
    mswitch<-which(diff(month)!=0)
    yswitch<-which(diff(year)!=0) 
    switch<-sort(unique(c(mswitch,yswitch))) 

    ## boundaries 
    abline(v=mswitch,col="gray")  
    abline(v=yswitch,col="gray",lwd=3)  

    ## month labels  
    mloc<-(c(0,mswitch)+c(mswitch,nrow(TPS)))/2
    mlab<- c("J","F","M","A","M","J",
	     "J","A","S","O","N","D")[month[round(mloc)]]
    axis(1, (c(0,mswitch)+c(mswitch,nrow(TPS)))/2,labels=mlab)

    ## temperature
    segments(1:nrow(TPS),TPS$TMAX,1:nrow(TPS),TPS$TMIN,col=tcols[istat]) 
    points(1:nrow(TPS),TPS$TAVG,col=tcols[istat],cex=.5,pch=16) 

    ## rain
    points(1:nrow(TPS),rep(min(TPDATA[,3:5])-(9/5)^(units=="F"),nrow(TPS)),
	    col=rcols[istat], pch=15 ) 
    abline(h=min(TPDATA[,3:5])-.25*(9/5)^(units=="F")) 

    ## year and station  
    jans<-mloc[ month[round(mloc)]==1  ]
    if(length(jans)>0){text(jans,max(TPDATA[,3:5]),year[jans],adj=.5)} else 
                      { text( 15,max(TPDATA[,3:5]),year[1],adj=.5) } 

  }

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/weatherStats.r
\name{weatherStats}
\alias{weatherStats}
\title{Weather Station Statistics}
\usage{
weatherStats(
  FAAID,
  year = NULL,
  startDate = NULL,
  endDate = NULL,
  trimNA = TRUE,
  maxTries = 5,
  tpause = 3,
  verbose = TRUE
)
}
\arguments{
\item{FAAID}{Federal Aviation Administration (FAA) station ID}

\item{year}{year for which data are to be downloaded}

\item{startDate}{start date in YY-MM-DD format}

\item{endDate}{end date in YY-MM-DD format}

\item{trimNA}{remove days with no data}

\item{maxTries}{maximum number of query attempts to the NCEI server}

\item{tpause}{time in seconds to pause between API requests}

\item{verbose}{a logical value indicating whether to report download progress}
}
\value{
a data frame with daily weather data
}
\description{
Download weather data from the National Centers for Environmental Information (NCEI) using their API.
}
\examples{
  
\donttest{
RDU<-weatherStats("RDU",startDate="2024-01-01",endDate="2025-12-31") 
SEA<-weatherStats("SEA",year=2025)   
}
 
dim(RDU) 
dim(SEA) 
colnames(SEA)
 
}
\author{
Peter Hoff
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tconvert.r
\name{c2f}
\alias{c2f}
\title{C to F temperature conversion}
\usage{
c2f(x)
}
\arguments{
\item{x}{temperature in Celsius}
}
\value{
temperature in Fahrenheit
}
\description{
Celsius to Fahrenheit temperature coversion
}
\author{
Peter Hoff
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.r
\docType{data}
\name{SEA}
\alias{SEA}
\title{SEA weather dataset}
\format{
A data frame with dates along the rows, variables along the columns.
}
\source{
Downloaded via the NCEI API <https://www.ncei.noaa.gov/support/access-search-service-api-user-documentation>.
}
\usage{
SEA
}
\description{
One year of weather data at SEA.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/weatherStats-package.r
\docType{package}
\name{weatherStats-package}
\alias{weatherStats-package}
\title{Airport Weather Station Statistics}
\description{
Download daily weather data recorded at airport weather stations using 
the National Centers for Environmental Information (NCEI) API 
<https://www.ncei.noaa.gov/support/access-search-service-api-user-documentation>.
}
\examples{
\donttest{
RDU<-weatherStats("RDU",startDate="2024-01-01",endDate="2025-12-31") 
SEA<-weatherStats("SEA",year=2025)   
}
W<-tpdata(RDU,SEA) 
 
colnames(W) 
dim(W) 
tpplot(W) 

}
\author{
Peter Hoff <peter.hoff@duke.edu>
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/isMonth.r
\name{isMonth}
\alias{isMonth}
\title{Month-specific date extraction}
\usage{
isMonth(dates, month)
}
\arguments{
\item{dates}{a vector of character dates in IS( 8601 format}

\item{month}{an integer from 1 to 12}
}
\value{
a vector of matching indices
}
\description{
Extract indices of a particular month from ISO 8601 format
}
\author{
Peter Hoff
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tconvert.r
\name{f2c}
\alias{f2c}
\title{F to C temperature conversion}
\usage{
f2c(y)
}
\arguments{
\item{y}{temperature in Fahrenheit}
}
\value{
temperature in Celsius
}
\description{
Fahrenheit to Celsius temperature coversion
}
\author{
Peter Hoff
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.r
\docType{data}
\name{RDU}
\alias{RDU}
\title{RDU weather dataset}
\format{
A data frame with dates along the rows, variables along the columns.
}
\source{
Derived from original data at https://example.org
}
\usage{
RDU
}
\description{
Two years of weather data at RDU.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tpdata.r
\name{tpdata}
\alias{tpdata}
\title{Temperature and precipitation data}
\usage{
tpdata(...)
}
\arguments{
\item{\ldots}{either a list or sequence of data matrices obtained from `weatherStats`}
}
\value{
a data frame
}
\description{
Extract and combine temperature and precipitation data
}
\examples{
W<-tpdata(RDU,SEA) 
 
colnames(W) 
dim(W) 

tpplot(W) 

}
\author{
Peter Hoff
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tpplot.r
\name{tpplot}
\alias{tpplot}
\title{Temperature and precipitation plot}
\usage{
tpplot(TPDATA, units = "F", rc = "col")
}
\arguments{
\item{TPDATA}{A data frame with temperature and precipitation data}

\item{units}{"F" or "C"}

\item{rc}{make multiple plots along columns ("col") or rows ("row")}
}
\value{
Invisibly returns \code{NULL}.
}
\description{
Plot temperature and precipitation data
}
\examples{
W<-tpdata(RDU,SEA) 
 
colnames(W) 
dim(W) 

tpplot(W) 

}
\author{
Peter Hoff
}
